%
%   Alternating Guided Filter (AGF)
%
%   F = AGF(image,sigmaSpatial,sigmaRange,niter) 
%   
%   Parameters: 
%   ===========
%   image        : input image, DOUBLE image, any # of channels.
%   sigmaSpatial : spatial sigma controlling the spatial weight of the bilateral filter. 
%   sigmaRange   : range sigma controlling the range weight of the bilateral filter. 
%   niter        : the number of iterations of rolling guidance.
%

%   Example:
%   ==========
%   image = im2double(imread('image.png'));
%   F = AGF(image,3,0.05,5);
%   imshow(F);
%
%   ===========
%   The code is based on the method described in the following paper:
%
%      "A. Toet (2016). Alternating Guided Image Filtering. Submitted to PeerJ Computer Science.  
%
%   ==========

function F = AGF(image,sigmaSpatial,sigmaRange,niter)

F = image;

for c=1:size(F,3)
	G = F(:,:,c).*0;
    for i=1:niter
		G = bilateralFilter(F(:,:,c),G,min(G(:)),max(G(:)),sigmaSpatial,sigmaRange); % = RGF
		G = bilateralFilter(G,F(:,:,c),min(F(:)),max(F(:)),sigmaSpatial,sigmaRange); % = SiR
		G = medfilt2(G,[3 3]);
	end
F(:,:,c) = G;	
end

			
