%
%   Rolling Guidance Filter (RGF)
%
%   F = RGF(image,sigmaSpatial,sigmaRange,niter) 
%   
%   Parameters: 
%   ===========
%   image        : input image, DOUBLE image, any # of channels.
%   sigmaSpatial : spatial sigma controlling the spatial weight of the bilateral filter. 
%   sigmaRange   : range sigma controlling the range weight of the bilateral filter. 
%   niter        : the number of iterations of rolling guidance.
%
%
%   Example:
%   ==========
%   image = im2double(imread('image.png'));
%   F = RGF(image,3,0.05,5);
%   imshow(F);
%
%   ===========
%   The code is based on the method described in the following paper:
%
%      "Zhang, Q., Shen, X., Xu, L. & Jia, J. (2014). Rolling guidance filter. 
%       In:  D. Fleet, T. Pajdla, B. Schiele & T. Tuytelaars (Eds.), 
%       Computer Vision - ECCV 2014: 13th European Conference,  Part III  (pp. 815-830). 
%       Cham, Switzerland: Springer International Publishing. DOI 10.1007/978-3-319-10578-9_53"
%
%       See also: http://www.cs.cuhk.edu.hk/~leojia/projects/rollguidance/ 
%
%   ==========


function F = RGF(image,sigmaSpatial,sigmaRange,niter)

F = image.*0; 

for c=1:size(image,3)
	for i=1:niter
        G = F(:,:,c);
        F(:,:,c) = bilateralFilter(image(:,:,c),G,min(G(:)),max(G(:)),sigmaSpatial,sigmaRange);
    end
end

