%
%   Smooth and iteratively Restore Filter (SiR) with median filtering
%
%   F = SiRmed(image,sigmaSpatial,sigmaRange,niter) 
%   
%   Parameters: 
%   ===========
%   image        : input image, DOUBLE image, any # of channels.
%   sigmaSpatial : spatial sigma controlling the spatial weight of the bilateral filter. 
%   sigmaRange   : range sigma controlling the range weight of the bilateral filter. 
%   niter        : the number of iterations of rolling guidance.
%

%   Example
%   ==========
%   image = im2double(imread('image.png'));
%   F = SiR(image,3,0.05,5);
%   imshow(F);
%
%   ===========
%   The code is based on the method described in the following paper:
%
%      "Kniefacz, P. & Kropatsch, W. (2015). Smooth and iteratively Restore: A simple and fast 
%       edge-preserving smoothing model. In:  S. Hegenbart, R. Kwitt & A. Uhl (Eds.), 
%       Proceedings of The 39th Annual Workshop of the Austrian Association for Pattern Recognition (OAGM 2015),  
%       eprint arXiv:1505.06702  (pp. 1-9).  
%
%   ==========

function F = SiRmed(image,sigma,range,niter)

F = image;
blur = imgaussfilt(F,sigma);

for c=1:size(F,3)
	G = blur(:,:,c);
    for i=1:niter
		G = bilateralFilter(G,F(:,:,c),min(F(:)),max(F(:)),sigma,range); 
		G = medfilt2(G,[3 3]);
	end	
F(:,:,c) = G;	
end

			
